Method: Zip::File.foreach

Defined in:
lib/zip/file.rb

.foreach(zip_file_name, &block) ⇒ Object

Iterates over the contents of the ZipFile. This is more efficient than using a ZipInputStream since this methods simply iterates through the entries in the central directory structure in the archive whereas ZipInputStream jumps through the entire archive accessing the local entry headers (which contain the same information as the central directory).



171
172
173
174
175
# File 'lib/zip/file.rb', line 171

def foreach(zip_file_name, &block)
  ::Zip::File.open(zip_file_name) do |zip_file|
    zip_file.each(&block)
  end
end